package com.hichina.main.back.hichinamainback.controller;

import com.hichina.main.back.hichinamainback.auth.AuthType;
import com.hichina.main.back.hichinamainback.auth.AuthUserInfo;
import com.hichina.main.back.hichinamainback.auth.GoogleAuthenticator;
import com.hichina.main.back.hichinamainback.config.CustomAuthenticationProvider;
import com.hichina.main.back.hichinamainback.mapper.UserMapper;
import com.hichina.main.back.hichinamainback.model.DTO.*;
import com.hichina.main.back.hichinamainback.model.User;
import com.hichina.main.back.hichinamainback.utils.FacebookAccessTokenValidator;
import com.hichina.main.back.hichinamainback.utils.HttpUtils;
import com.hichina.main.back.hichinamainback.utils.UserUtil;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.function.Consumer;

@RestController
@RequestMapping("/api/public/login")
public class PublicLoginController {

    private static final Logger LOGGER = LoggerFactory.getLogger(PublicLoginController.class);

    private static final String GRAPH_API_URL = "https://graph.facebook.com/v14.0/me?access_token=";

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private FacebookAccessTokenValidator facebookAccessTokenValidator;

    @Autowired
    private GoogleAuthenticator googleAuthenticator;


    @Autowired
    private Environment env;

    @PostMapping("/prereg-facebook")
    public HichinaResponse preRegisterFacebook(@RequestBody PreregisterFacebookReqDTO request) {
        HichinaResponse ret = new HichinaResponse();
        if (facebookAccessTokenValidator.validateAccessToken(request.getAccessToken())) {
            // if already exist, just update some property, otherwise, register one and set corresponding login type to facebook
            generateOrUpdateUser(request.getEmail(), (user -> {
                user.setFacebookId(request.getFacebookId());
                user.setUsername(request.getName());
                user.setProfileImageUrl(request.getProfileImageUrl());
                user.setLoginType(AuthType.FACEBOOK.getType());
            }));
            ret.setOk(true);
            ret.setMessage("Succeed pre register or update facebook user");
        } else {
            ret.setOk(false);
            ret.setMessage("Access token not valid for facebook login");
        }
        return ret;
    }

    @PutMapping("/type")
    public HichinaResponse updateLoginType(@RequestBody UpdateLoginTypeRequestDTO request) {
        HichinaResponse ret = new HichinaResponse();
        User user = UserUtil.getUserByEmail(userMapper, request.getEmail());
        if (user == null) {
            ret.setOk(false);
            ret.setMessage(String.format("User does not exist for email: %s", request.getEmail()));
        } else {
            user.setLoginType(request.getLoginType());
            userMapper.update(user);
            ret.setOk(true);
            ret.setMessage(String.format("Succeed updating login type for user : %s", request.getEmail()));
        }
        return ret;
    }

    @PostMapping("/google")
    public ResponseEntity<HiChinaResult<AuthResultDTO>> googleAuth(HttpServletRequest request) {
        String idToken = HttpUtils.parseBearerToken(request);
        if (idToken == null) {
            return ResponseEntity.badRequest().body(HiChinaResult.error("Invalid token"));
        }

        try {
            AuthUserInfo authUserInfo = googleAuthenticator.verify(idToken);
            String email = authUserInfo.getEmail();
            generateOrUpdateUser(email, (user -> {
                user.setGoogleId(authUserInfo.getUserId());
                user.setUsername(authUserInfo.getUsername());
                user.setProfileImageUrl(authUserInfo.getProfileImageUrl());
                user.setLoginType(AuthType.GOOGLE.getType());
            }));

            return ResponseEntity.ok(HiChinaResult.ok(AuthResultDTO.of(authUserInfo.getEmail())));
        } catch (Exception e) {
            LOGGER.error("google auth failed: {}", e.getMessage(), e);
            return ResponseEntity.badRequest().body(HiChinaResult.error("Google auth failed"));
        }

    }

    private User generateOrUpdateUser(String email, Consumer<User> userSetter) {
        User user = UserUtil.getUserByEmail(userMapper, email);
        if (user == null) {
            //register new user
            user = new User();
            user.setCreatedTime(new Date());
            user.setSalt(CustomAuthenticationProvider.generateSalt());
            user.setPassword("");
            user.setEmail(email);
            user.setPwdCode(-1);
            user.setUserId(java.util.UUID.randomUUID().toString());
            userSetter.accept(user);
            userMapper.insert(user);
        } else {
            userSetter.accept(user);
            userMapper.update(user);
        }
        return user;
    }
}
